<?php

namespace Guru\Offer\Http\Controllers;

use App\Http\Controllers\AdminApiController as AdminController;
use Guru\Offer\Http\Requests\OfferAdminApiRequest;
use Guru\Offer\Interfaces\OfferRepositoryInterface;
use Guru\Offer\Models\Offer;

/**
 * Admin API controller class.
 */
class OfferAdminApiController extends AdminController
{
    /**
     * Initialize offer controller.
     *
     * @param type OfferRepositoryInterface $offer
     *
     * @return type
     */
    public function __construct(OfferRepositoryInterface $offer)
    {
        $this->repository = $offer;
        parent::__construct();
    }

    /**
     * Display a list of offer.
     *
     * @return json
     */
    public function index(OfferAdminApiRequest $request)
    {
        $offers  = $this->repository
            ->setPresenter('\\Guru\\Offer\\Repositories\\Presenter\\OfferListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $offers['code'] = 2000;
        return response()->json($offers) 
                         ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display offer.
     *
     * @param Request $request
     * @param Model   Offer
     *
     * @return Json
     */
    public function show(OfferAdminApiRequest $request, Offer $offer)
    {
        $offer         = $offer->presenter();
        $offer['code'] = 2001;
        return response()->json($offer)
                         ->setStatusCode(200, 'SHOW_SUCCESS');;

    }

    /**
     * Show the form for creating a new offer.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(OfferAdminApiRequest $request, Offer $offer)
    {
        $offer         = $offer->presenter();
        $offer['code'] = 2002;
        return response()->json($offer)
                         ->setStatusCode(200, 'CREATE_SUCCESS');

    }

    /**
     * Create new offer.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(OfferAdminApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $offer          = $this->repository->create($attributes);
            $offer          = $offer->presenter();
            $offer['code']  = 2004;

            return response()->json($offer)
                             ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
;
        }
    }

    /**
     * Show offer for editing.
     *
     * @param Request $request
     * @param Model   $offer
     *
     * @return json
     */
    public function edit(OfferAdminApiRequest $request, Offer $offer)
    {
        $offer         = $offer->presenter();
        $offer['code'] = 2003;
        return response()-> json($offer)
                        ->setStatusCode(200, 'EDIT_SUCCESS');;
    }

    /**
     * Update the offer.
     *
     * @param Request $request
     * @param Model   $offer
     *
     * @return json
     */
    public function update(OfferAdminApiRequest $request, Offer $offer)
    {
        try {

            $attributes = $request->all();

            $offer->update($attributes);
            $offer         = $offer->presenter();
            $offer['code'] = 2005;

            return response()->json($offer)
                             ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the offer.
     *
     * @param Request $request
     * @param Model   $offer
     *
     * @return json
     */
    public function destroy(OfferAdminApiRequest $request, Offer $offer)
    {

        try {

            $t = $offer->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('offer::offer.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
